( cd usr/lib ; rm -rf libpbm.so.1 )
( cd usr/lib ; ln -sf libpbm.so.1.0.0 libpbm.so.1 )
( cd usr/lib ; rm -rf libpbm.so )
( cd usr/lib ; ln -sf libpbm.so.1.0.0 libpbm.so )
( cd usr/lib ; rm -rf libpgm.so.1 )
( cd usr/lib ; ln -sf libpgm.so.1.0.0 libpgm.so.1 )
( cd usr/lib ; rm -rf libpgm.so )
( cd usr/lib ; ln -sf libpgm.so.1.0.0 libpgm.so )
( cd usr/lib ; rm -rf libppm.so.1 )
( cd usr/lib ; ln -sf libppm.so.1.0.0 libppm.so.1 )
( cd usr/lib ; rm -rf libppm.so )
( cd usr/lib ; ln -sf libppm.so.1.0.0 libppm.so )
( cd usr/lib ; rm -rf libpnm.so.1 )
( cd usr/lib ; ln -sf libpnm.so.1.0.0 libpnm.so.1 )
( cd usr/lib ; rm -rf libpnm.so )
( cd usr/lib ; ln -sf libpnm.so.1.0.0 libpnm.so )
( cd usr/lib ; rm -rf libfbm.so.1 )
( cd usr/lib ; ln -sf libfbm.so.1.0.0 libfbm.so.1 )
( cd usr/lib ; rm -rf libfbm.so )
( cd usr/lib ; ln -sf libfbm.so.1.0.0 libfbm.so )
( cd usr/lib ; rm -rf librle.so.1 )
( cd usr/lib ; ln -sf librle.so.1.0.0 librle.so.1 )
( cd usr/lib ; rm -rf librle.so )
( cd usr/lib ; ln -sf librle.so.1.0.0 librle.so )
